<?php
/**
 * @version 1.5 stable $Id: controller.php 1896 2014-05-01 18:12:25Z ggppdk $
 * @package Joomla
 * @subpackage FLEXIcontent
 * @copyright (C) 2009 Emmanuel Danan - www.vistamedia.fr
 * @license GNU/GPL v2
 * 
 * FLEXIcontent is a derivative work of the excellent QuickFAQ component
 * @copyright (C) 2008 Christoph Lukes
 * see www.schlu.net for more information
 *
 * FLEXIcontent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('legacy.controller.legacy');
use Joomla\String\StringHelper;

/**
 * FLEXIcontent Component Controller
 *
 * @package Joomla
 * @subpackage FLEXIcontent
 * @since 1.0
 */
class FlexicontentController extends JControllerLegacy
{
	function __construct()
	{
		parent::__construct();
		$params = JComponentHelper::getParams('com_flexicontent');
		$print_logging_info = $params->get('print_logging_info');
		if ( $print_logging_info )  global $fc_run_times;
		
		jimport('cms.version.version');
		$jversion = new JVersion;
		
		$config_saved = !FLEXI_J16GE ? $params->get('flexi_section', 0) : $params->get('flexi_cat_extension', 0);
		//$config_saved = $config_saved && $params->get('search_mode', 0);  // an Extra configuration check
		
		// If configuration not saved REDIRECT TO DASHBOARD VIEW (will ask to save or import)
		$view = JRequest::getVar('view');
		if($view && !$config_saved) {
			$link 	= 'index.php?option=com_flexicontent';
			$this->setRedirect($link);   // we do not message since this will be displayed by template of the view ...
		}
		$session = JFactory::getSession();
		
		// GET POSTINSTALL tasks from session variable AND IF NEEDED re-evaluate it
		// NOTE, POSTINSTALL WILL NOT LET USER USE ANYTHING UNTIL ALL TASKS ARE COMPLETED
		$postinst_integrity_ok = $session->get('flexicontent.postinstall');
		$recheck_aftersave = $session->get('flexicontent.recheck_aftersave');
		
		$collation_version = $session->get('flexicontent.collation_version');
		if ($collation_version != $jversion->getShortVersion())  $postinst_integrity_ok = NULL;
		
		//$valArray = array(false => 'false', true => 'true', null=>'null');
		//echo  "postinst_integrity_ok: " . (isset($valArray[$postinst_integrity_ok])  ?  $valArray[$postinst_integrity_ok]  :  $postinst_integrity_ok) ."<br/>\n";
		//echo  "recheck_aftersave: " . (isset($valArray[$recheck_aftersave])  ?  $valArray[$recheck_aftersave]  :  $recheck_aftersave) ."<br/>\n";
		
		$format	= strtolower(JRequest::getCmd('format', 'html'));
		if ($format == 'html')
		{
			if ( $postinst_integrity_ok===NULL || $postinst_integrity_ok===false || $recheck_aftersave )
			{
				// NULL mean POSTINSTALL tasks has not been checked YET (current PHP user session),
				// false means it has been checked during current session, but has failed one or more tasks
				// In both cases we must evaluate the POSTINSTALL tasks,  and set the session variable
				if ( $print_logging_info ) $start_microtime = microtime(true);
				$postinst_integrity_ok = $this->getPostinstallState();
				//echo  "set postinst_integrity_ok: " . (isset($valArray[$postinst_integrity_ok])  ?  $valArray[$postinst_integrity_ok]  :  $postinst_integrity_ok) ."<br/>\n";
				$session->set('flexicontent.postinstall', $postinst_integrity_ok);
				$session->set('unbounded_count', false, 'flexicontent');  // indicate to item manager to recheck unbound items
				if ( $print_logging_info ) @$fc_run_times['post_installation_tasks'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
			}
			
			// SET recheck_aftersave FLAG to indicate rechecking of (a) post installation tasks AND (b) integrity checks after configuration save or article importing
			if ($config_saved) {
				$session->set('flexicontent.recheck_aftersave', !$postinst_integrity_ok);
				//echo  "set recheck_aftersave: " . (isset($valArray[!$postinst_integrity_ok])  ?  $valArray[!$postinst_integrity_ok]  :  !$postinst_integrity_ok) ."<br/>\n";
			} else {
				$session->set('flexicontent.recheck_aftersave', true);
				//echo  "set recheck_aftersave: true" ."<br/>\n";
			}
			
			if ( $print_logging_info ) $start_microtime = microtime(true);
			
			// GET ALLPLGPUBLISH task from session variable AND IF NEEDED re-evaluate it
			// NOTE, we choose to have this separate from REQUIRED POSTINSTALL tasks,
			// because WE DON'T WANT TO FORCE the user to enable all plugins but rather recommend it
			$allplgpublish = $session->get('flexicontent.allplgpublish');
			if(($allplgpublish===NULL) || ($allplgpublish===false)) {
				// NULL means ALLPLGPUBLISH task has not been checked YET (current PHP user session),
				// false means it has been checked during current session but has failed
				// In both cases we must evaluate the ALLPLGPUBLISH task,  and set the session variable
				$model = $this->getModel('flexicontent');
				$allplgpublish = $model->getAllPluginsPublished();
				$session->set('flexicontent.allplgpublish', $allplgpublish);
			}
			
			if($view && in_array($view, array('items', 'item', 'types', 'type', 'categories', 'category', 'fields', 'field', 'tags', 'tag', 'archive', 'filemanager', 'templates', 'stats', 'search', 'import')) && !$postinst_integrity_ok) {
				$msg = JText::_( 'FLEXI_PLEASE_COMPLETE_POST_INSTALL' );
				$link 	= 'index.php?option=com_flexicontent';
				$this->setRedirect($link, $msg);
			} else if ($postinst_integrity_ok && $config_saved) {
				$this->checkDirtyFields();
			}
		}
		
		// Register Extra task
		$this->registerTask( 'apply'								, 'save' );
		$this->registerTask( 'applyacl'							, 'saveacl' );
		$this->registerTask( 'createmenuitems'			, 'createMenuItems' );
		$this->registerTask( 'createdefaultype'			, 'createDefaultType' );
		$this->registerTask( 'createdefaultfields'	, 'createDefaultFields' );
		$this->registerTask( 'publishplugins'				, 'publishplugins' );
		$this->registerTask( 'addmcatitemrelations'	, 'addMcatItemRelations' );
		$this->registerTask( 'createlangcolumn'			, 'createLangColumn' );
		$this->registerTask( 'createdbindexes'			, 'createDBindexes' );
		$this->registerTask( 'createversionstbl'		, 'createVersionsTable' );
		$this->registerTask( 'populateversionstbl'	, 'populateVersionsTable' );
		$this->registerTask( 'createauthorstbl'			, 'createauthorstable' );
		$this->registerTask( 'updateitemcounting'   , 'updateItemCountingData' );
		$this->registerTask( 'deletedeprecatedfiles', 'deleteDeprecatedFiles' );
		$this->registerTask( 'cleanupoldtables'			, 'cleanupOldTables' );
		$this->registerTask( 'addcurrentversiondata', 'addCurrentVersionData' );
		$this->registerTask( 'langfiles'						, 'processLanguageFiles' );
		
		$task = JRequest::getVar('task');
		if (is_string($task) && $task=="translate") {
			JRequest::setVar('task', 'copy');
			JRequest::setVar('copy_behaviour', 'translate');
		}
		
		if ( $print_logging_info ) @$fc_run_times['initialize_component'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
	}
	
	function processLanguageFiles() 
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));
		
		$code 	= JRequest::getVar('code', 'en-GB');
		$method = JRequest::getVar('method', '');
		$name = JRequest::getVar('name', '');
		$email = JRequest::getVar('email', '');
		$web = JRequest::getVar('web', '');
		$message = JRequest::getVar('message', '');
		
		$formparams = array();
		$formparams['name'] 	= $name;
		$formparams['email'] 	= $email;
		$formparams['web'] 		= $web;
		$formparams['message'] 	= $message;
		
		$model 	= $this->getModel('flexicontent');		

		$missing = $model->processLanguageFiles($code, $method, $formparams);
		
		if (is_array($missing) && $method != 'zip') {
			if (@$missing['admin']) {
				echo '<h3>'.JText::_('Folder: administrator/languages/').$code.'/</h3>';
				foreach ($missing['admin'] as $a) {
					echo '<p>'.$code.'.'.$a.'.ini'.(($method == 'create') ? ' <span class="lang-success">created</span>' : ' <span class="lang-fail">is missing</span>').'</p>';
				}
			}
			if (@$missing['site']) {
				echo '<h3>'.JText::_('Folder: languages/').$code.'/</h3>';
				foreach ($missing['site'] as $s) {
					echo '<p>'.$code.'.'.$s.'.ini'.(($method == 'create') ? ' <span class="lang-success">created</span>' : ' <span class="lang-fail">is missing</span>').'</p>';
				}
			}
			if ($method != 'create') {
				echo '<style>#missing {display:block;}</style>';
			}
		} else {
			echo $missing;
			echo '<style>#missing {display:none;}</style>';
		}
	}

	function getPostinstallState()
	{
		$params = JComponentHelper::getParams('com_flexicontent');
		$print_logging_info = $params->get('print_logging_info');
		if ( $print_logging_info ) {
			global $fc_run_times;
			$start_microtime = microtime(true);
		}
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$model  = $this->getModel('flexicontent');
		$model->checkCollations();
		$model->install_3rdParty_plugins();
		
		$params = JComponentHelper::getParams('com_flexicontent');
		$use_versioning = $params->get('use_versioning', 1);
		if ( $print_logging_info ) @$fc_run_times['checking_postinstall_task_init'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [checking_postinstall_task_init: %.2f s] ', $fc_run_times['checking_postinstall_task_init']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existtype 			= $model->getExistType();
		if ( $print_logging_info ) @$fc_run_times['getExistType'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistType: %.2f s] ', $fc_run_times['getExistType']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existmenuitems	= $model->getExistMenuItems();
		if ( $print_logging_info ) @$fc_run_times['getExistMenuItems'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistMenuItems: %.2f s] ', $fc_run_times['getExistMenuItems']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existfields 		= $model->getExistFields();
		if ( $print_logging_info ) @$fc_run_times['getExistFields'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistFields: %.2f s] ', $fc_run_times['getExistFields']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existfplg 			= $model->getExistFieldsPlugins();
		if ( $print_logging_info ) @$fc_run_times['getExistFieldsPlugins'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistFieldsPlugins: %.2f s] ', $fc_run_times['getExistFieldsPlugins']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existseplg 		= $model->getExistSearchPlugin();
		if ( $print_logging_info ) @$fc_run_times['getExistSearchPlugin'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistSearchPlugin: %.2f s] ', $fc_run_times['getExistSearchPlugin']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existsyplg 		= $model->getExistSystemPlugin();
		if ( $print_logging_info ) @$fc_run_times['getExistSystemPlugin'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistSystemPlugin: %.2f s] ', $fc_run_times['getExistSystemPlugin']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existcats        = !$model->getItemsNoCat();
		if ( $print_logging_info ) @$fc_run_times['getItemsNoCat'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getItemsNoCat: %.2f s] ', $fc_run_times['getItemsNoCat']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existlang				= $model->getExistLanguageColumns() && !$model->getItemsNoLang();
		if ( $print_logging_info ) @$fc_run_times['getItemsNoLang'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getItemsNoLang: %.2f s] ', $fc_run_times['getItemsNoLang']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existversions 		= $model->getExistVersionsTable();
		if ( $print_logging_info ) @$fc_run_times['getExistVersionsTable'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistVersionsTable: %.2f s] ', $fc_run_times['getExistVersionsTable']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existversionsdata= !$use_versioning || $model->getExistVersionsPopulated();
		if ( $print_logging_info ) @$fc_run_times['getExistVersionsPopulated'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistVersionsPopulated: %.2f s] ', $fc_run_times['getExistVersionsPopulated']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existauthors 		= $model->getExistAuthorsTable();
		if ( $print_logging_info ) @$fc_run_times['getExistAuthorsTable'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistAuthorsTable: %.2f s] ', $fc_run_times['getExistAuthorsTable']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$cachethumb				= $model->getCacheThumbChmod();
		if ( $print_logging_info ) @$fc_run_times['getCacheThumbChmod'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getCacheThumbChmod: %.2f s] ', $fc_run_times['getCacheThumbChmod']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$deprecatedfiles	= $model->getDeprecatedFiles();
		if ( $print_logging_info ) @$fc_run_times['getDeprecatedFiles'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getDeprecatedFiles: %.2f s] ', $fc_run_times['getDeprecatedFiles']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$nooldfieldsdata	= $model->getNoOldFieldsData();
		if ( $print_logging_info ) @$fc_run_times['getNoOldFieldsData'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getNoOldFieldsData: %.2f s] ', $fc_run_times['getNoOldFieldsData']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$missingversion		= true; //!$use_versioning || !$model->checkCurrentVersionData();
		if ( $print_logging_info ) @$fc_run_times['checkCurrentVersionData'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [checkCurrentVersionData: %.2f s] ', $fc_run_times['checkCurrentVersionData']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$existdbindexes = $model->getExistDBindexes();
		if ( $print_logging_info ) @$fc_run_times['getExistDBindexes'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getExistDBindexes: %.2f s] ', $fc_run_times['getExistDBindexes']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$itemcountingdok  = $model->getItemCountingDataOK();
		if ( $print_logging_info ) @$fc_run_times['getItemCountingDataOK'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [getItemCountingDataOK: %.2f s] ', $fc_run_times['getItemCountingDataOK']/1000000);
	
		if ( $print_logging_info ) $start_microtime = microtime(true);
		$initialpermission = $model->checkInitialPermission();
		if ( $print_logging_info ) @$fc_run_times['checkInitialPermission'] += round(1000000 * 10 * (microtime(true) - $start_microtime)) / 10;
		//printf('<br/>-- [checkInitialPermission: %.2f s] ', $fc_run_times['checkInitialPermission']/1000000);
		
		// Check if old field positions were converted
		$model->convertOldFieldsPositions();
		
		//echo "(!$existtype) || (!$existmenuitems) || (!$existfields) ||<br>";
		//echo "     (!$existfplg) || (!$existseplg) || (!$existsyplg) ||<br>";
		//echo "     (!$existcats)  || (!$existlang) || (!$existdbindexes) || (!$itemcountingdok) || (!$existversions) || (!$existversionsdata) || (!$existauthors) || (!$cachethumb) ||<br>";
		//echo "     (!$deprecatedfiles) || (!$nooldfieldsdata) || (!$missingversion) ||<br>";
		//echo "     (!$initialpermission)<br>";
	
		// Display POST installation tasks if any task-check fails (returns false)
		$postinst_integrity_ok = true;
		if (
			!$existtype || !$existmenuitems || !$existfields ||
			//!$existfplg || !$existseplg || existsyplg ||
			!$existcats || !$existlang || !$existversions || !$existversionsdata || !$existauthors ||
			!$deprecatedfiles || !$nooldfieldsdata || !$missingversion || !$cachethumb ||
			!$existdbindexes || !$itemcountingdok || !$initialpermission
		) {
			$postinst_integrity_ok = false;
		}
		return $postinst_integrity_ok;
	}


	/**
	 * Display the view
	 */
	function display($cachable = false, $urlparams = false)
	{
		parent::display();
	}
	
	
	/**
	 * Saves the acl file
	 *
	 */
	function saveacl()
	{

		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		// Initialize some variables
		$app     = JFactory::getApplication();
		$option  = JRequest::getVar('option');
		$filename    = JRequest::getVar('filename', '', 'post', 'cmd');
		$filecontent = JRequest::getVar('filecontent', '', '', '', JREQUEST_ALLOWRAW);

		if (!$filecontent) {
			$app->redirect('index.php?option='.$option, JText::_( 'FLEXI_OPERATION_FAILED' ).': '.JText::_( 'FLEXI_CONTENT_EMPTY' ));
		}

		// Set FTP credentials, if given
		jimport('joomla.client.helper');
		JClientHelper::setCredentialsFromRequest('ftp');
		$ftp = JClientHelper::getCredentials('ftp');

		$file = JPATH_SITE.DS.'components'.DS.'com_flexicontent'.DS.'classes'.DS.$filename;

		// Try to make the acl file writeable
		if (!$ftp['enabled'] && JPath::isOwner($file) && !JPath::setPermissions($file, '0755')) {
			JError::raiseNotice('SOME_ERROR_CODE', JText::_( 'FLEXI_COULD_NOT_MAKE_ACL_FILE_WRITABLE' ));
		}

		jimport('joomla.filesystem.file');
		$return = JFile::write($file, $filecontent);

		// Try to make the acl file unwriteable
		if (!$ftp['enabled'] && JPath::isOwner($file) && !JPath::setPermissions($file, '0555')) {
			JError::raiseNotice('SOME_ERROR_CODE', JText::_( 'FLEXI_COULD_NOT_MAKE_ACL_FILE_UNWRITABLE' ));
		}

		if ($return)
		{
			$task = JRequest::getVar('task');
			switch($task)
			{
				case 'applyacl' :
					$app->redirect('index.php?option='.$option.'&view=editacl', JText::_( 'FLEXI_ACL_FILE_SUCCESSFULLY_ALTERED' ));
					break;

				case 'saveacl'  :
				default         :
					$app->redirect('index.php?option='.$option, JText::_( 'FLEXI_ACL_FILE_SUCCESSFULLY_ALTERED' ) );
					break;
			}
		} else {
			$app->redirect('index.php?option='.$option, JText::_( 'FLEXI_OPERATION_FAILED' ).': '.JText::sprintf('FLEXI_FAILED_TO_OPEN_FILE_FOR_WRITING', $file));
		}
	}
	
	/**
	 * Method to create default type : article
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createDefaultType()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();

		$query 	=	'
		INSERT INTO `#__flexicontent_types`
			(id, asset_id, name, alias, published, checked_out, checked_out_time, access, attribs)
		VALUES
		(
			1, 0, "Article", "article", 1, 0, "0000-00-00 00:00:00", 1,
			\'{"ilayout":"default","hide_maintext":"0","hide_html":"0","maintext_label":"","maintext_desc":"","comments":"","top_cols":"two","bottom_cols":"two","allow_jview":"1"}\'
		)
		';
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		}
		
		$query 	=	'
		INSERT INTO `#__flexicontent_fields_type_relations`
			(`field_id`,`type_id`,`ordering`)
		VALUES
			(1,1,1), (2,1,2), (3,1,3), (4,1,4), (5,1,5), (6,1,6), (7,1,7), (8,1,8), (9,1,9), (10,1,10), (11,1,11), (12,1,12), (13,1,13), (14,1,14)
		';
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	/**
	 * Method to create default menu items used for SEF links
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createMenuItems()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();
		if (FLEXI_J16GE) {
			$db->setQuery("SELECT extension_id FROM #__extensions WHERE element='com_flexicontent' AND type='component' ");
		} else {
			$db->setQuery("SELECT id FROM #__components WHERE admin_menu_link='option=com_flexicontent'");
		}
		$flexi_comp_id = $db->loadResult();	
		
		$db->setQuery("DELETE FROM #__menu_types WHERE menutype='flexihiddenmenu' ");	
		$db->execute();
		
		$db->setQuery("INSERT INTO #__menu_types (`menutype`,`title`,`description`) ".
			"VALUES ('flexihiddenmenu', 'FLEXIcontent Hidden Menu', 'A hidden menu to host Flexicontent needed links')");
		$db->execute();
		
		$db->setQuery("DELETE FROM #__menu WHERE menutype='flexihiddenmenu' ");	
		$db->execute();
		
		$query 	=	"INSERT INTO #__menu ("
			."`menutype`,`title`,`alias`,`path`,`link`,`type`,`published`,`parent_id`,`component_id`,`level`,"
			."`checked_out`,`checked_out_time`,`browserNav`,`access`,`params`,`lft`,`rgt`,`home`, `language`"
		.") VALUES ("
			."'flexihiddenmenu','Content','content_page','content_page','index.php?option=com_flexicontent&view=flexicontent','component',1,1,$flexi_comp_id,1,"
			."0,'0000-00-00 00:00:00',0,1,'rootcat=0',0,0,0,'*'"
		.")";
		
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		}
		
		// Save the created menu item as default_menu_itemid for the component
		$cparams = JComponentHelper::getParams('com_flexicontent');
		$cparams->set('default_menu_itemid', $db->insertid());
		$cparams_str = $cparams->toString();
		
		$flexi = JComponentHelper::getComponent('com_flexicontent');
		$query = 'UPDATE '. (FLEXI_J16GE ? '#__extensions' : '#__components')
				. ' SET params = ' . $db->Quote($cparams_str)
				. ' WHERE '. (FLEXI_J16GE ? 'extension_id' : 'id') .'='. $flexi->id
				;
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
		
		// This is necessary as extension data are cached ... and just above we updated the component parameters -manually- (and (also added menu item)
		$cache = JFactory::getCache();
		$cache->clean( '_system' );
	}
	
	
	/**
	 * Method to create default fields data
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createDefaultFields()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();
		
		$query 	=	'
		INSERT INTO `#__flexicontent_fields`
			(`id`,`field_type`,`name`,`label`,`description`,`isfilter`,`iscore`,`issearch`,`isadvsearch`,`untranslatable`,`formhidden`,`valueseditable`,`edithelp`,`positions`,`published`,`attribs`,`checked_out`,`checked_out_time`,`access`,`ordering`)
		VALUES
			(1,"maintext","text","Description","Main description text (introtext/fulltext)",0,1,1,0,0,0,0,1,"description.items.default",1,\'{"display_label":"0","trigger_onprepare_content":"1"}\',0,"0000-00-00 00:00:00",1,2),
			(2,"created","created","Created","Date this item was created",0,1,1,0,0,0,0,1,"top.items.default\nabove-description-line1-nolabel.category.blog",1,\'{"display_label":"1","date_format":"DATE_FORMAT_LC1","custom_date":"","pretext":"","posttext":""}\',0,"0000-00-00 00:00:00",1,3),
			(3,"createdby","created_by","Created by","User (owner) who created this item",0,1,1,0,0,0,0,1,"top.items.default\nabove-description-line1-nolabel.category.blog",1,\'{"display_label":"1","pretext":"","posttext":""}\',0,"0000-00-00 00:00:00",1,4),
			(4,"modified","modified","Last modified","Date this item was last modified",0,1,1,0,0,0,0,1,"top.items.default",1,\'{"display_label":"1","date_format":"DATE_FORMAT_LC1","custom_date":"","pretext":"","posttext":""}\',0,"0000-00-00 00:00:00",1,5),
			(5,"modifiedby","modified_by","Revised by","User who last modified this item",0,1,1,0,0,0,0,1,"top.items.default",1,\'{"display_label":"1","pretext":"","posttext":""}\',0,"0000-00-00 00:00:00",1,6),
			(6,"title","title","Title","Item title",0,1,1,0,0,0,0,1,"",1,\'{"display_label":"1"}\',0,"0000-00-00 00:00:00",1,1),
			(7,"hits","hits","Hits","Number of hits",0,1,1,0,0,0,0,1,"",1,\'{"display_label":"1","pretext":"","posttext":"views"}\',0,"0000-00-00 00:00:00",1,7),
			(8,"type","document_type","Document type","Document type",0,1,1,0,0,0,0,1,"",1,\'{"display_label":"1","pretext":"","posttext":""}\',0,"0000-00-00 00:00:00",1,8),
			(9,"version","version","Version","Latest version number",0,1,1,0,0,0,0,1,"",1,\'{"display_label":"1","pretext":"","posttext":""}\',0,"0000-00-00 00:00:00",1,9),
			(10,"state","state","State","Publication status",0,1,1,0,0,0,0,1,"",1,\'{"display_label":"1"}\',0,"0000-00-00 00:00:00",1,10),
			(11,"voting","voting","Voting","Voting buttons",0,1,1,0,0,0,0,1,"top.items.default\nabove-description-line2-nolabel.category.blog",1,\'{"display_label":"1","dimension":"16","image":"components/com_flexicontent/assets/images/star-small.png"}\',0,"0000-00-00 00:00:00",1,11),
			(12,"favourites","favourites","Favourites","Add to favourites button",0,1,1,0,0,0,0,1,"top.items.default\nabove-description-line2-nolabel.category.blog",1,\'{"display_label":"1"}\',0,"0000-00-00 00:00:00",1,12),
			(13,"categories","categories","Categories","Categories this item is assigned to",0,1,1,0,0,0,0,1,"top.items.default\nunder-description-line1.category.blog",1,\'{"display_label":"1","separatorf":"2"}\',0,"0000-00-00 00:00:00",1,13),
			(14,"tags","tags","Tags","Tags assigned to this item",0,1,1,0,0,0,0,1,"top.items.default\nunder-description-line2.category.blog",1,\'{"display_label":"1","separatorf":"2"}\',0,"0000-00-00 00:00:00",1,14)
		';
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
	}

	/**
	 * Publish FLEXIcontent plugins
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function publishplugins()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$format = JRequest::getVar('format', '');
		$db = JFactory::getDBO();
		
		$query	= 'UPDATE '. (FLEXI_J16GE ? '#__extensions' : '#__plugins')
				. ' SET '. (FLEXI_J16GE ? 'enabled' : 'published') .' = 1'
				. ' WHERE '. (FLEXI_J16GE ? ' `type`= ' . $db->Quote('plugin') : '1')
				. ' AND (folder = ' . $db->Quote('flexicontent_fields')
				. ' OR element = ' . $db->Quote('flexisearch')
				. ' OR element = ' . $db->Quote('flexisystem')
				. ' OR element = ' . $db->Quote('flexiadvsearch')
				. ' OR element = ' . $db->Quote('flexiadvroute')
				. ')'
				;
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			if ($format == 'raw') {
				echo '<span class="install-notok"></span>';
				if ($db->getErrorNum()) echo $db->getErrorMsg();
				jexit();
			} else {
				$db_err_msg = $db->getErrorNum() ? ' :<br/>'.$db->getErrorMsg() : '';
				JError::raiseNotice(1, JText::_('FLEXI_COULD_NOT_PUBLISH_PLUGINS') . $db_err_msg);
				return false;
			}
		} else {
			if ($format == 'raw') {
				echo '<span class="install-ok"></span>';
			} else {
				return true;
			}
		}
	}

	/**
	 * Set phpThumb cache permissions
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function cachethumbchmod()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$format		= JRequest::getVar('format', '');
		// PhpThumb cache directory
		$phpthumbcache 	= JPath::clean(JPATH_SITE.DS.'components'.DS.'com_flexicontent'.DS.'librairies'.DS.'phpthumb'.DS.'cache');
		$success = JPath::setPermissions($phpthumbcache, '0600', '0700');
		if (!$success) {
			if ($format == 'raw') {
				echo '<span class="install-notok"></span>';
				jexit();
			} else {
				JError::raiseNotice(1, JText::_( 'FLEXI_COULD_NOT_SET_PHPTHUMB_PERMS' ));
				return false;
			}
		} else {
			if ($format == 'raw') {
				echo '<span class="install-ok"></span>';
			} else {
				return true;
			}
		}
	}
	
	
	/**
	 * Method to sync language between Joomla and FLEXIcontent tables
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function syncItemsLang()
	{
		$db = JFactory::getDBO();
		
		// This should be match items that come from J1.5 upgrade only
		// and copy the J1.5 language from items_ext table into the content table
		$query 	= 'UPDATE #__content AS i'
					.' LEFT JOIN #__flexicontent_items_ext as ie ON i.id=ie.item_id'
				. ' SET i.language = ie.language'
				. ' WHERE ie.language<>"" AND ('
				. '  i.language="" OR (i.language="*" AND i.language<>ie.language)'
				. ' )'
				;
		$db->setQuery($query);
		$result1 = $db->execute();
		
		// Sync language of items_ext table using the language from Joomla content table
		$query 	= 'UPDATE #__flexicontent_items_ext AS ie'
					.' LEFT JOIN #__content as i ON i.id=ie.item_id'
				. ' SET ie.language = i.language'
				. ' WHERE i.language<>ie.language'
				;
		$db->setQuery($query);
		$result1 = $db->execute();
	}
	
	
	
	/**
	 * Method to set the default site language the items with no language
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function setItemsDefaultLang($lang)
	{
		$db = JFactory::getDBO();
		
		// Set default language for items that do not have their language set
		$query 	= 'UPDATE #__flexicontent_items_ext'
				. ' SET language = ' . $db->Quote($lang)
				. ' WHERE language = ""'
				;
		$db->setQuery($query);
		$result1 = $db->execute();
		
		$query 	= 'UPDATE #__flexicontent_items_tmp'
				. ' SET language = ' . $db->Quote($lang)
				. ' WHERE language = ""'
				;
		$db->setQuery($query);
		$result1a = $db->execute();
		
		$query 	= 'UPDATE #__content'
				. ' SET language = ' . $db->Quote($lang)
				. ' WHERE language = ""'
				;
		$db->setQuery($query);
		$result2 = $db->execute();
		
		return $result1 && $result1a && $result2;
	}
		
	
	/**
	 * Method to set the default site language the items with no language
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function addmcatitemrelations()
	{
		$db = JFactory::getDBO();
		
		// 1st: remove orphan relations
		$query = "DELETE rel.*"
			." FROM #__flexicontent_cats_item_relations AS rel"
			." LEFT JOIN #__content AS i ON i.id = rel.itemid"
			." WHERE i.id IS NULL";
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		}
		
		// 2nd: add missing main category relations
		$subquery 	= "SELECT i.catid, i.id, 0 FROM #__flexicontent_items_ext as ie "
			. " JOIN #__content as i ON i.id=ie.item_id "
			. " LEFT JOIN #__flexicontent_cats_item_relations as rel ON rel.catid=i.catid AND i.id=rel.itemid "
			. " WHERE rel.catid IS NULL";
		
		// Set default language for items that do not have their language set
		$query 	= 'INSERT INTO #__flexicontent_cats_item_relations'
			.' (catid, itemid, ordering) '.$subquery;
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	/**
	 * Method to create the language datas
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createLangColumn()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();
		$nullDate	= $db->getNullDate();
		
		// Add language column
		$columns = $db->getTableColumns('#__flexicontent_items_ext');
		$language_col = array_key_exists('language', $columns) ? true : false;
		if(!$language_col)
		{
			$query 	=	"ALTER TABLE #__flexicontent_items_ext ADD `language` VARCHAR( 11 ) NOT NULL DEFAULT '' AFTER `type_id`" ;
			$db->setQuery($query);
			$result_lang_col = $db->execute();
			if (!$result_lang_col) echo "Cannot add language column<br>";
		} else $result_lang_col = true;
		
		// Add translation group column
		$lang_parent_id_col = array_key_exists('lang_parent_id', $columns) ? true : false;
		if(!$lang_parent_id_col)
		{
			$query 	=	"ALTER TABLE #__flexicontent_items_ext ADD `lang_parent_id` INT NOT NULL DEFAULT 0 AFTER `language`" ;
			$db->setQuery($query);
			$result_tgrp_col = $db->execute();
			if (!$result_tgrp_col) echo "Cannot add translation group column<br>";
		} else $result_tgrp_col = true;
		
		// Add default language for items that do not have one, and add translation group to items that do not have one set
		$model = $this->getModel('flexicontent');
		if ($model->getItemsNoLang())
		{
			// 1. copy language from __flexicontent_items_ext table into __content
			$this->syncItemsLang();
			
			// 2. then for those that are still empty, add site default language to the language field if empty
			$lang = flexicontent_html::getSiteDefaultLang();
			$result_items_default_lang = $this->setItemsDefaultLang($lang);
			if (!$result_items_default_lang) echo "Cannot set default language or set default translation group<br>";
		} else $result_items_default_lang = true;
		
		
		$query 	=	"
			INSERT INTO `#__associations` (`id`, `context`, `key`)
				SELECT DISTINCT ie.item_id, 'com_content.item', ie.lang_parent_id
				FROM `#__flexicontent_items_ext` AS ie
				JOIN `#__flexicontent_items_ext` AS j ON ie.lang_parent_id = j.lang_parent_id AND ie.item_id<>j.item_id
				WHERE ie.lang_parent_id <> 0
			ON DUPLICATE KEY UPDATE id=id";
		$db->setQuery($query);
		try {
			$convert_assocs = $db->execute();
			$query 	=	"UPDATE `#__flexicontent_items_ext` SET lang_parent_id = 0";
			$db->setQuery($query);
			$clear_assocs = $db->execute();
		}
		catch (Exception $e) {
			echo "Cannot convert FLEXIcontent associations to Joomla associations<br>";
			JError::raiseWarning( 500, $e->getMessage() );
			$convert_assocs = $clear_assocs = false;
		}
		
		if ( !$result_lang_col
			|| !$result_tgrp_col
			|| !$result_items_default_lang
			|| !$convert_assocs
			|| !$clear_assocs
		) {
			echo '<span class="install-notok"></span>';
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	
	/**
	 * Method to create missing DB indexes
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createDBindexes()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));
		
		$db = JFactory::getDBO();
		$nullDate	= $db->getNullDate();
		
		$model  = $this->getModel('flexicontent');

		$update_queries = array();
		$missing_indexes = $model->getExistDBindexes($check_only=false, $update_queries);

		if ( !empty($missing_indexes) )
		{
			$app = JFactory::getApplication();
			
			foreach($missing_indexes as $tblname => $indexnames)
			{
				$index_cmds = array();
				$index_cmds['indexdrop'] = array();   // !! important add 'indexdrop' to 'index_cmds', before 'indexadd' !!
				$index_cmds['indexadd'] = array();
				if ( isset($indexnames['__indexing_started__']) ) continue;
				foreach($indexnames as $indexname => $iconf)
				{
					if (!is_array($iconf))
					{
						$indexlen = $iconf ? "(".$iconf.")" : "";
						$index_cmds['indexadd'][] = " ADD INDEX " . $indexname . "(`".$indexname."`" .$indexlen. ")";
					}
					else
					{
						$indexdrop = !empty($iconf['custom_drop'])
							? $iconf['custom_drop']
							: '';
						if ($indexdrop)
						{
							$index_cmds['indexdrop'][] = $indexdrop;
						}

						$indexadd  = !empty($iconf['custom_add'])
							? $iconf['custom_add']
							: ' ADD INDEX ' . $indexname;
						if ($indexadd && !empty($iconf['cols']))
						{
							$_col_list = array();
							foreach($iconf['cols'] as $indexcol => $len)
							{
								$indexlen  = $len ? "(".$len.")" : "";
								$_col_list[] = "`".$indexcol."`" .$indexlen;
							}
							$index_cmds['indexadd'][]   = $indexadd . "(". implode(", ", $_col_list) .")";
						}
					}
				}
				
				// For MyISAM the table is copied for the purpose of adding indexes and then old table is dropped
				// so it is better to add ALL table indexes via single command ?
				// For InnoDB in MySQL 5.1+, table is not copied so these when adding indexes it is better to have InnoDB tables
				if ( !empty($index_cmds) )
				{
					$file = JPATH_SITE.DS.'tmp'.DS.'tbl_indexes_'.$tblname;
					$file_contents = "".time();
					JFile::write($file, $file_contents);

					if ( isset($update_queries[$tblname]) )
					{
						$db->setQuery($update_queries[$tblname]);
						try { $result = $db->execute(); }
						catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below

						if (!$result)
						{
							echo '<span class="install-notok"></span>';
							if ($db->getErrorNum()) echo $db->getErrorMsg();
							jexit();
						}
					}

					// Allow dropping of duplicate rows using ALTER IGNORE TABLE
					//$db->setQuery('SET session old_alter_table=1');
					//$db->execute();

					foreach($index_cmds as $index_type => $index_clause)
					{
						$query  = 'ALTER TABLE #__' . $tblname . ' ';  //'ALTER IGNORE TABLE #__' . $tblname . ' ';
						$query .= implode(', ', $index_clause);
						$db->setQuery($query);

						try { $result = $db->execute(); }
						catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below

						if (!$result && $index_type!='indexdrop')
						{
							echo '<span class="install-notok"></span>';
							if ($db->getErrorNum()) echo $db->getErrorMsg();
							jexit();
						}
					}
					JFile::delete($file);
				}
			}
			echo '<span class="install-ok"></span>';
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	
	/**
	 * Method to update content into non-indexed columns
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function updateItemCountingData()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));
		
		$db = JFactory::getDBO();
		$cache_tbl = "#__flexicontent_items_tmp";
		
		// Truncate the table, this will handle redudant columns too
		$db->setQuery('TRUNCATE TABLE '.$cache_tbl);
		$db->execute();
		$model = $this->getModel('items');
		$result = $model->updateItemCountingData($rows = false);
		
		if ( !$result ) {
			echo '<span class="install-notok"></span>';
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	
	/**
	 * Method to create the versions table
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createVersionsTable()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();
		$nullDate	= $db->getNullDate();

		$query 	= " CREATE TABLE IF NOT EXISTS #__flexicontent_versions (
	  				`id` int(11) unsigned NOT NULL auto_increment,
					`item_id` int(11) unsigned NOT NULL default '0',
					`version_id` int(11) unsigned NOT NULL default '0',
					`comment` mediumtext NOT NULL,
					`created` datetime NOT NULL default '0000-00-00 00:00:00',
					`created_by` int(11) unsigned NOT NULL default '0',
					`state` int(3) NOT NULL default '0',
					PRIMARY KEY  (`id`),
					KEY `version2item` (`item_id`,`version_id`)
					) ENGINE=MyISAM CHARACTER SET `utf8` COLLATE `utf8_general_ci`"
					;
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
			if ($db->getErrorNum()) echo $db->getErrorMsg();
			jexit();
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	/**
	 * Method to create the authors table
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function createAuthorsTable()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();
		$nullDate	= $db->getNullDate();

		$query 	= " CREATE TABLE IF NOT EXISTS #__flexicontent_authors_ext (
  				`user_id` int(11) unsigned NOT NULL,
  				`author_basicparams` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL,
  				`author_catparams` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL,
					PRIMARY KEY  (`user_id`)
					) ENGINE=MyISAM CHARACTER SET `utf8` COLLATE `utf8_general_ci`"
					;
		$db->setQuery($query);
		
		try { $result = $db->execute(); }
		catch (Exception $e) { $result = false; } // suppress exception in case of SQL error, we will print it below
		
		if (!$result) {
			echo '<span class="install-notok"></span>';
		} else {
			echo '<span class="install-ok"></span>';
		}
	}
	
	/**
	 * Method to handle the versions data and to populate the new beta4 versions table
	 * From #__flexicontent_items_versions to #__flexicontent_versions
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function populateVersionsTable()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db = JFactory::getDBO();
		$nullDate	= $db->getNullDate();

		$query 	= 'SELECT item_id, version FROM #__flexicontent_items_versions'
				. ' WHERE field_id = 2'
				. ' ORDER BY item_id, version'
				;
		$db->setQuery($query);
		$cvs = $db->loadObjectList();

		$query 	= 'SELECT * FROM #__flexicontent_items_versions'
				. ' WHERE field_id IN ( 2,3,4,5 )'
				. ' ORDER BY item_id, version, field_id'
				;
		$db->setQuery($query);
		$fvs = $db->loadObjectList();

		$_fvs = array();
		foreach ($fvs as $fv)
		{
			$_fvs[$fv->item_id][$fv->version][$fv->field_id] = $fv;
		}
		$fvs = $_fvs;

		$versioned_fids = array(2=>'created', 3=>'created_by', 4=>'modified', 5=>'modified_by');
		for ($i=0; $i<count($cvs); $i++)
		{
			foreach($versioned_fids as $fid => $fname)
			{
				if (isset($fvs[$cvs[$i]->item_id][$cvs[$i]->version][$fid]))
				{
					$cvs[$i]->$fname = $fvs[$cvs[$i]->item_id][$cvs[$i]->version][$fid]->value;
				}
				else $cvs[$i]->$fname = 0;
			}
		}

		$versions = new stdClass();
		$n = 0;
		foreach ($cvs as $cv)
		{
			$versions->$n = new stdClass();
			$versions->$n->item_id    = $cv->item_id;
			$versions->$n->version_id = $cv->version;
			$versions->$n->comment    = '';
			$versions->$n->created    = (isset($cv->modified) && ($cv->modified != $nullDate)) ? $cv->modified : $cv->created;
			$versions->$n->created_by = (isset($cv->modified_by) && $cv->modified_by) ? $cv->modified_by : $cv->created_by;
			$versions->$n->state      = 1;
			$db->insertObject('#__flexicontent_versions', $versions->$n);
			$n++;
		}

		echo '<span class="install-ok"></span>';
	}

	/**
	 * Method to check if the files from beta3 still exist in the category and item view
	 *
	 * @access public
	 * @return	boolean	True on success
	 */
	function deleteDeprecatedFiles()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$model = $this->getModel('flexicontent');
		$deprecated = null;
		$model->getDeprecatedFiles($deprecated);
		
		jimport('joomla.filesystem.file');
		foreach ($deprecated as $dir => $dirfiles) {
			foreach ($dirfiles as $file) {
				JFile::delete($dir.DS.$file);
			}
		}
		
		if ($model->getDeprecatedFiles()) {
			echo '<span class="install-ok"></span>';
		} else {
			echo '<span class="install-notok"></span>';
		}
	}
	
	
	/**
	 * Method to delete old core fields data in the fields_items_relations table
	 * Delete also old versions fields data
	 * Alter value fields to mediumtext in order to store large items
	 * 
	 * @access	public
	 * @return	boolean	True on success
	 * @since 1.5
	 */
	function cleanupOldTables()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$db  = JFactory::getDBO();
		$app = JFactory::getApplication();

		$queries 	= array();
		// alter some table field types
		$queries[] 	= "ALTER TABLE #__flexicontent_fields_item_relations CHANGE `value` `value` MEDIUMTEXT" ;
		$queries[] 	= "ALTER TABLE #__flexicontent_items_versions CHANGE `value` `value` MEDIUMTEXT" ;
		$queries[] 	= "ALTER TABLE #__flexicontent_items_ext"
			."  CHANGE `search_index` `search_index` MEDIUMTEXT"
			.", CHANGE `sub_items` `sub_items` TEXT"
			.", CHANGE `sub_categories` `sub_categories` TEXT"
			.", CHANGE `related_items` `related_items` TEXT"
			;
		foreach ($queries as $query) {
			$db->setQuery($query);
			$db->execute();
		}
		$query = "SELECT id,version,created,created_by FROM #__content " . (!FLEXI_J16GE ? "WHERE sectionid='".FLEXI_SECTION."'" : "");
		$db->setQuery($query);
		$rows = $db->loadObjectList();
		
		//$jcorefields = flexicontent_html::getJCoreFields();
		$add_cats = true;
		$add_tags = true;
		$clean_database = true;
		
		// For all items not having the current version, add it
		$last_versions = FLEXIUtilities::getLastVersions();
		foreach($rows as $row)
		{
			$lastversion = @$last_versions[$row->id]['version'];
			
			if($row->version > $lastversion)
			{
				// Get field values of the current item version
				$query = "SELECT f.id,fir.value,f.field_type,f.name,fir.valueorder,fir.suborder,f.iscore "
						." FROM #__flexicontent_fields_item_relations as fir"
						." JOIN #__flexicontent_fields as f on f.id=fir.field_id "
						." WHERE fir.item_id=".$row->id." AND f.iscore=0";  // old versions stored categories & tags into __flexicontent_fields_item_relations
				$db->setQuery($query);
				$fields = $db->loadObjectList();
				
				// Delete old data
				if ($clean_database && $fields) {
					$query = 'DELETE FROM #__flexicontent_fields_item_relations WHERE item_id = '.$row->id;
					$db->setQuery($query);
					$db->execute();
				}
				
				// Add the 'maintext' field to the fields array for adding to versioning table
				$f = new stdClass();
				$f->id					= 1;
				$f->iscore			= 1;
				$f->valueorder	= 1;
				$f->suborder    = 1;
				$f->field_type	= "maintext";
				$f->name				= "text";
				$f->value				= $row->introtext;
				if ( StringHelper::strlen($row->fulltext) > 1 ) {
					$f->value .= '<hr id="system-readmore" />' . $row->fulltext;
				}
				if(substr($f->value, 0, 3)!="<p>") {
					$f->value = "<p>".$f->value."</p>";
				}
				$fields[] = $f;

				// Add the 'categories' field to the fields array for adding to versioning table
				$query = "SELECT catid FROM #__flexicontent_cats_item_relations WHERE itemid='".$row->id."';";
				$db->setQuery($query);
				$categories = FLEXI_J16GE ? $db->loadColumn() : $db->loadResultArray();
				if (!$categories || !count($categories))
				{
					$categories = array($catid = $row->catid);
					$query = "INSERT INTO #__flexicontent_cats_item_relations VALUES('$catid','".$row->id."', '0');";
					$db->setQuery($query);
					$db->execute();
				}
				$f = new stdClass();
				$f->id					= 13;
				$f->iscore			= 1;
				$f->valueorder	= 1;
				$f->suborder    = 1;
				$f->version		= (int)$row->version;
				$f->value		= serialize($categories);
				if ($add_cats) $fields[] = $f;
				
				// Add the 'tags' field to the fields array for adding to versioning table
				$query = "SELECT tid FROM #__flexicontent_tags_item_relations WHERE itemid='".$row->id."';";
				$db->setQuery($query);
				$tags = $db->loadColumn();
				$f = new stdClass();
				$f->id					= 14;
				$f->iscore			= 1;
				$f->valueorder	= 1;
				$f->suborder    = 1;
				$f->version		= (int)$row->version;
				$f->value		= serialize($tags);
				if ($add_tags) $fields[] = $f;

				// Add field values to field value versioning table
				foreach($fields as $field)
				{
					// add the new values to the database 
					$obj = new stdClass();
					$obj->field_id   = $field->id;
					$obj->item_id    = $row->id;
					$obj->valueorder = $field->valueorder;
					$obj->suborder   = $field->suborder;
					$obj->version    = (int)$row->version;
					$obj->value      = $field->value;
					//echo "version: ".$obj->version.",fieldid : ".$obj->field_id.",value : ".$obj->value.",valueorder : ".$obj->valueorder.",suborder : ".$obj->suborder."<br />";
					//echo "inserting into __flexicontent_items_versions<br />";
					$db->insertObject('#__flexicontent_items_versions', $obj);

					if (!$field->iscore)
					{
						unset($obj->version);
						//echo "inserting into __flexicontent_fields_item_relations<br />";
						$db->insertObject('#__flexicontent_fields_item_relations', $obj);
						flexicontent_db::setValues_commonDataTypes($obj);
					}
					//$searchindex 	.= @$field->search;
				}
				
				// **********************************************************************************
				// Add basic METADATA of current item version (kept in table #__flexicontent_versions)
				// **********************************************************************************
				$v = new stdClass();
				$v->item_id    = (int)$row->id;
				$v->version_id = (int)$row->version;
				$v->created    = ($row->modified && ($row->modified != $nullDate)) ? $row->modified : $row->created;
				$v->created_by = $row->created_by;
				$v->comment    = '';
				//echo "inserting into __flexicontent_versions<br />";
				$db->insertObject('#__flexicontent_versions', $v);
			}
		}
		
		$queries 	= array();
		// delete unused records
		// 1,'maintext',  2,'created',  3,'createdby',  4,'modified',  5,'modifiedby',  6,'title',  7,'hits'
		// 8,'type',  9,'version',  10,'state',   11,'voting',  12,'favourites',  13,'categories',  14,'tags'
		$queries[] 	= "DELETE FROM #__flexicontent_fields_item_relations WHERE field_id < 15" ;
		$queries[] 	= "DELETE FROM #__flexicontent_items_versions WHERE field_id IN ( 7, 9, 11, 12 )" ;
		
		foreach ($queries as $query) {
			$db->setQuery($query);
			$db->execute();
		}

		$catscache = JFactory::getCache('com_flexicontent_cats');
		$catscache->clean();

		$model = $this->getModel('flexicontent');
		if ($model->getNoOldFieldsData()) {
			echo '<span class="install-ok"></span>';
		} else {
			echo '<span class="install-notok"></span>';
			jexit();
		}
	}
	
	function addCurrentVersionData()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$model = $this->getModel('flexicontent');
		if ($model->addCurrentVersionData()) {
			echo '<span class="install-ok"></span>';
		} else {
			echo '<span class="install-notok"></span>';
			jexit();
		}
	}
	
	function initialPermission() {
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));

		$model = $this->getModel('flexicontent');
		if ($model->initialPermission()) {
			echo '<span class="install-ok"></span>';
		} else {
			echo '<span class="install-notok"></span>';
			jexit();
		}
	}

	function checkDirtyFields() {
		if (JRequest::getVar('task')!='' || JRequest::getVar('format')!='' || JRequest::getVar('tmpl')!='') return;
		$perms = FlexicontentHelperPerm::getPerm();
		if ( !$perms->CanFields ) return;
		
		$db = JFactory::getDBO();
		
		// GET fields having dirty field properties, NOTE: a dirty field property means that search index must be updated,
		// even if the field was unpublished, because the field values may still exists in the search index for some items
		
		$query = 'SELECT COUNT(*) '
			. ' FROM #__flexicontent_fields'
			. ' WHERE (issearch=-1 || issearch=2)'  // Regardless publication state
			;
		$db->setQuery($query);
		$dirty_basic = $db->loadResult();
		
		$query = 'SELECT COUNT(*) '
			. ' FROM #__flexicontent_fields'
			. ' WHERE (isadvsearch=-1 OR isadvsearch=2 OR isadvfilter=-1 OR isadvfilter=2)'  // Regardless publication state
			;
		$db->setQuery($query);
		$dirty_advanced = $db->loadResult();
		
		if ($dirty_basic)    JError::raiseNotice( 403, JText::sprintf( 'FLEXI_ALERT_UPDATE_SINDEX_BASIC', $dirty_basic, ' href="index.php?option=com_flexicontent&view=search&layout=indexer&tmpl=component&indexer=basic" class="btn" onclick="var url = jQuery(this).attr(\'href\'); fc_showDialog(url, \'fc_modal_popup_container\', 0, 550, 350, function(){window.location.reload(false)}); return false;" ') );
		if ($dirty_advanced) JError::raiseNotice( 403, JText::sprintf( 'FLEXI_ALERT_UPDATE_SINDEX_ADVANCED', $dirty_advanced, ' href="index.php?option=com_flexicontent&view=search&layout=indexer&tmpl=component&indexer=advanced" class="btn" onclick="var url = jQuery(this).attr(\'href\'); fc_showDialog(url, \'fc_modal_popup_container\', 0, 550, 350, function(){window.location.reload(false)}); return false;" ') );
	}
	
	
	function fversioncompare()
	{
		// Check for request forgeries
		JSession::checkToken('request') or jexit(JText::_('JINVALID_TOKEN'));
		@ob_end_clean();
		JRequest::setVar('layout', 'fversion');
		parent::display();
		exit;
	}


	function call_extfunc()
	{
		flexicontent_ajax::call_extfunc();
	}
}
?>
